<h1 align="center">AtomicGo | constraints</h1>

<p align="center">
<img src="https://img.shields.io/endpoint?url=https%3A%2F%2Fatomicgo.dev%2Fapi%2Fshields%2Fconstraints&style=flat-square" alt="Downloads">

<a href="https://github.com/atomicgo/constraints/releases">
<img src="https://img.shields.io/github/v/release/atomicgo/constraints?style=flat-square" alt="Latest Release">
</a>

<a href="https://codecov.io/gh/atomicgo/constraints" target="_blank">
<img src="https://img.shields.io/github/actions/workflow/status/atomicgo/constraints/go.yml?style=flat-square" alt="Tests">
</a>

<a href="https://codecov.io/gh/atomicgo/constraints" target="_blank">
<img src="https://img.shields.io/codecov/c/gh/atomicgo/constraints?color=magenta&logo=codecov&style=flat-square" alt="Coverage">
</a>

<a href="https://codecov.io/gh/atomicgo/constraints">
<!-- unittestcount:start --><img src="https://img.shields.io/badge/Unit_Tests-126-magenta?style=flat-square" alt="Unit test count"><!-- unittestcount:end -->
</a>

<a href="https://opensource.org/licenses/MIT" target="_blank">
<img src="https://img.shields.io/badge/License-MIT-yellow.svg?style=flat-square" alt="License: MIT">
</a>
  
<a href="https://goreportcard.com/report/github.com/atomicgo/constraints" target="_blank">
<img src="https://goreportcard.com/badge/github.com/atomicgo/constraints?style=flat-square" alt="Go report">
</a>   

</p>

---

<p align="center">
<strong><a href="https://pkg.go.dev/atomicgo.dev/constraints#section-documentation" target="_blank">Documentation</a></strong>
|
<strong><a href="https://github.com/atomicgo/atomicgo/blob/main/CONTRIBUTING.md" target="_blank">Contributing</a></strong>
|
<strong><a href="https://github.com/atomicgo/atomicgo/blob/main/CODE_OF_CONDUCT.md" target="_blank">Code of Conduct</a></strong>
</p>

---

<p align="center">
  <img src="https://raw.githubusercontent.com/atomicgo/atomicgo/main/assets/header.png" alt="AtomicGo">
</p>

<p align="center">
<table>
<tbody>
</tbody>
</table>
</p>
<h3  align="center"><pre>go get atomicgo.dev/constraints</pre></h3>
<p align="center">
<table>
<tbody>
</tbody>
</table>
</p>

<!-- gomarkdoc:embed:start -->

<!-- Code generated by gomarkdoc. DO NOT EDIT -->

# constraints

```go
import "atomicgo.dev/constraints"
```

Package constraints provides constraints for type parameters aka. generics in Go.

This is a more comprehensive drop\-in replacement for golang.org/x/exp/constraints.

## Index

- [type Addable](<#Addable>)
- [type Comparable](<#Comparable>)
- [type Complex](<#Complex>)
- [type Float](<#Float>)
- [type Integer](<#Integer>)
- [type Number](<#Number>)
- [type Numeric](<#Numeric>)
- [type Orderable](<#Orderable>)
- [type Ordered](<#Ordered>)
- [type Signed](<#Signed>)
- [type Unsigned](<#Unsigned>)


<a name="Addable"></a>
## type [Addable](<https://github.com/atomicgo/constraints/blob/main/comparison.go#L14-L16>)

Addable is a constraint that matches any type that can be added with the \+ operator.

```go
type Addable interface {
    Number | ~string
}
```

<a name="Comparable"></a>
## type [Comparable](<https://github.com/atomicgo/constraints/blob/main/comparison.go#L4-L6>)

Comparable is a constraint that matches any type that can be compared with the == and \!= operators.

```go
type Comparable interface {
    comparable
}
```

<a name="Complex"></a>
## type [Complex](<https://github.com/atomicgo/constraints/blob/main/numbers.go#L9-L11>)

Complex is a constraint that matches any complex number type.

```go
type Complex interface {
    ~complex64 | ~complex128
}
```

<a name="Float"></a>
## type [Float](<https://github.com/atomicgo/constraints/blob/main/numbers.go#L21-L23>)

Float is a constraint that matches any floating point number type.

```go
type Float interface {
    ~float32 | ~float64
}
```

<a name="Integer"></a>
## type [Integer](<https://github.com/atomicgo/constraints/blob/main/numbers.go#L26-L28>)

Integer is a constraint that matches any integer type.

```go
type Integer interface {
    Signed | Unsigned
}
```

<a name="Number"></a>
## type [Number](<https://github.com/atomicgo/constraints/blob/main/numbers.go#L16-L18>)

Number is a constraint that matches any real number type. The Number constraint does not include Complex numbers, as those are not ordered. If you also need Complex numbers, use the Numeric constraint.

```go
type Number interface {
    Float | Integer
}
```

<a name="Numeric"></a>
## type [Numeric](<https://github.com/atomicgo/constraints/blob/main/numbers.go#L4-L6>)

Numeric is a constraint that matches any numeric type, including complex numbers.

```go
type Numeric interface {
    Number | Complex
}
```

<a name="Orderable"></a>
## type [Orderable](<https://github.com/atomicgo/constraints/blob/main/comparison.go#L9-L11>)

Orderable is a constraint that matches any type that can be ordered with the \<, \<=, \>, and \>= operators.

```go
type Orderable interface {
    Integer | Float | ~string
}
```

<a name="Ordered"></a>
## type [Ordered](<https://github.com/atomicgo/constraints/blob/main/numbers.go#L31-L33>)

Ordered is a constraint that matches any ordered type.

```go
type Ordered interface {
    Integer | Float | ~string
}
```

<a name="Signed"></a>
## type [Signed](<https://github.com/atomicgo/constraints/blob/main/numbers.go#L36-L38>)

Signed is a constraint that matches any signed integer type.

```go
type Signed interface {
    ~int | ~int8 | ~int16 | ~int32 | ~int64
}
```

<a name="Unsigned"></a>
## type [Unsigned](<https://github.com/atomicgo/constraints/blob/main/numbers.go#L41-L43>)

Unsigned is a constraint that matches any unsigned integer type.

```go
type Unsigned interface {
    ~uint | ~uint8 | ~uint16 | ~uint32 | ~uint64 | ~uintptr
}
```

Generated by [gomarkdoc](<https://github.com/princjef/gomarkdoc>)


<!-- gomarkdoc:embed:end -->

---

> [AtomicGo.dev](https://atomicgo.dev) &nbsp;&middot;&nbsp;
> with ❤️ by [@MarvinJWendt](https://github.com/MarvinJWendt) |
> [MarvinJWendt.com](https://marvinjwendt.com)
